package cohy.correlator;
import 	 sep.operator.*;
import   jam.vector.*;
import   juice.vector.*;
import	 rsf.vector.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the correlation between an input data set and a signal data set.
 * The input and signal data are unravelled into two traces. The two traces 
 * are then correlated. The correlation is 1 if and only if the two data 
 * sets are identical. 
 * 
 * @see: TraceCorrelator 
 */
public class Correlation {

  static float correlate(float[] data, float[] sgnl) {
    float corr = 0.f; float nrmi = 0.f; float nrmj = 0.f;
    if (data.length != sgnl.length) 
      System.err.println("Error: dimensional mismatch");
    int n0 = data.length;
    for (int i0 = 0; i0 < n0; i0++) {
      corr += data[i0] * sgnl[i0];
      nrmi += data[i0] * data[i0]; 
      nrmj += sgnl[i0] * sgnl[i0];
    }
    return corr / (float) Math.sqrt(nrmi * nrmj); 
  }
  static float semblance(float[] data, float[] sgnl) {
    float corr = 0.f; float nrmi = 0.f; float nrmj = 0.f;
    if (data.length != sgnl.length) 
      System.err.println("Error: dimensional mismatch");
    int n0 = data.length;
    for (int i0 = 0; i0 < n0; i0++) {
      corr += data[i0] * sgnl[i0];
      nrmi += data[i0] * data[i0]; 
      nrmj += sgnl[i0] * sgnl[i0];
    }
    return 2 * corr / (nrmi*nrmi + nrmj*nrmj); 
  }
}





