package cohy.datstat;

import juice.util.*;

/**
 * This attempts to find the simplest representation 
 * of the given floating numbers. The given data set may had been converted 
 * from a more restricted representation, such as bytes or hexa-decimal 
 * representations. This is not finished. Input should be 1-D Rsf. Use 
 * rsf.util.Randomselector if necessary. 
 */
public class FloatAnalysis {
  public int base; 
  public int precision; 
  public int lower;
  public int upper;
  public int sign; 
  boolean isType;
  float   trueType;
  String  testType;
  int   exp;
  int[] digs; 
  float remainder;
  float num;
  float min,max;

  public static void main(String[] args) {
    FloatAnalysis machine = new FloatAnalysis();
    machine.setByteTest();
    machine.test(251.f); 
    System.out.println(machine.reportNumber()); 
    machine.test(5.f);
    System.out.println(machine.reportNumber()); 
    float[] d = {283.f, 144.f, 23.f, 200.f, 129.f, 128.f, 127.f, 230.5f};
    machine.test(d); 
    System.out.println(machine.reportArray()); 
  }

  public FloatAnalysis() {} 
  public FloatAnalysis(String testType, 
		       int base, int precision, int lower, int upper) {
    this.testType  = testType; 
    this.base      = base;
    this.precision = precision; 
    this.lower     = lower;
    this.upper     = upper;
    setTypeInit();
  }
  public void setHexaTest() {
    testType  = "Hexa Decimal"; 
    base      = 16;
    precision = 10; 
    lower     = -8;
    upper     =  8;
    setTypeInit();
  }
  public void setFloatTest() {
    testType  = "Typical Float"; 
    base      =   2;
    precision =  56; 
    lower     = -64;
    upper     =  64;
    setTypeInit();
  } 
  public void setByteTest() {
    testType  = "Byte"; 
    base      =   2;
    precision =   8; 
    lower     =   8;
    upper     =   8;
    setTypeInit();
  }
  public void setIBMTest() {
    testType  = "Byte"; 
    base      =   16;
    precision =   6; 
    lower     =  -64;
    upper     =   63;
    setTypeInit();
  }

  void setTypeInit() {
    min = (float)  Math.pow(base,lower-1);
    max = (float) (Math.pow(base,upper  ) * (1.f - Math.pow(base,-precision)));
  }

  void setNumberInit() {
    digs   = new int[precision];
    isType = true;
  }

  void test(float[] array) {
    int counter = 0; 
    for (int i=0; i<array.length; i++) {
      test(array[i]);
      if (isType) counter++; 
    }
    if (counter == array.length) isType = true ; 
    else                         isType = false; 
    trueType = 100.f * counter / array.length; 
  }

  void test(float num) {
    this.num = num; 
    setNumberInit();
    for (int i=0; i<digs.length; i++) digs[i] = 0;
    sign = (num > 0.f)? 1:-1;                               // sign     // 
    num  = sign * num;
    if (num < min || num > max) isType = false; 
    exp  = 1 + (int) (Math.log(num) / Math.log(base));      // exp      //
    num /= Math.pow(base,exp);                              // mantissa //
    for (int i=0; i<precision; i++) {
      digs[i]  = (int) (num * base);
      num      = num * base - digs[i];
    }
    remainder = num; 
    if(remainder != 0.f) isType = false;
  }

  String reportNumber() {
    StringBuffer buffy = new StringBuffer(400);
    buffy.append("\n" + testType + " represenatation\n");
    buffy.append("float number: " + num + " = "); 
    buffy.append(JuiceUtil.arrayToString(digs,",") + " * " + base +"^" + exp);
    if (isType) 
      buffy.append(    " of type " + testType);
    else 
      buffy.append(" not of type " + testType);

    buffy.append("\n min: " + min + " max: " + max); 
    return new String(buffy); 
  }

  String reportArray() {
    StringBuffer buffy = new StringBuffer(400);
    buffy.append(testType + " represenatation\n");
    buffy.append(testType + " Numbers: " + trueType + "% \n");
    if (isType) 
      buffy.append("   Fits "         + testType + " representation!!!\n");
    else 
      buffy.append("   Does not fit " + testType + " representation\n"); 
    return new String(buffy);
  }
}


