package cohy.datstat;

import java.util.Random; 
import juice.util.*;

class FloatGenerator {
  int   base;
  int   precision; 
  int   lower;
  int   upper;
  int   exp; 
  int   sign; 
  int[] digs; 

  FloatGenerator(int base, int precision, int lower, int upper) {
    this.base      = base; 
    this.precision = precision; 
    this.lower     = lower;
    this.upper     = upper;
    this.exp       = lower;
    this.digs      = new int[precision];
  }

  public static void main(String[] args) {
    boolean correct = true; 

    System.out.println("\n Golub's example:");
    FloatGenerator golub = new FloatGenerator(2,3,0,2); 
    golub.count();
    System.out.println("\n Binary  example:");
    FloatGenerator binary = new FloatGenerator(2,8,8,8); 
    binary.count(); 

    System.out.println("\n Random:"); 
    FloatGenerator binary2 = new FloatGenerator(2,8,8,8); 
    binary2.setRandomDigitals(); 
    binary2.report();

    Tester.exit(correct); 
  }

  public void count() { 
    for (int i = 0; i < digs.length; i++) digs[i] = 0; 
    for (exp = lower; exp <= upper    ; exp++){        // loop exponent //
      count(0); 
    }
  }

  void count(int pos) {
    if (pos == 0) {                                    // nonzero 1st digit //
      for (int i = 1; i < base; i++) {
	digs[pos] = i; 
	count(pos+1); 
      }
    }
    else if (pos+1 < digs.length) {
      for (int i = 0; i < base; i++) {
	digs[pos] = i; 
	count(pos+1); 
      }
    }
    else {
      for (int i = 0; i < base; i++) {
	digs[pos] = i;
	report(); 
      }
    }
  } 

  /** 
   * converts digits to float representation as fraction: 
   * @return Sum_{i=0}^{digs.length} (digs[i] * base^(-i-1)) 
   */  
  float digits(int[] digs) {
    float dd = 0; 
    for (int i  = 0; i < digs.length; i++) { 
      dd += digs[i] * (float) Math.pow(base,-i-1);
    } 
    return dd;
  }
  /** 
   * multiplies with exponent
   * @return dd * base^exp 
   */
  float valueOf(float dd) {
    return (dd * (float) Math.pow(base,exp)); 
  }

  void report() {
    StringBuffer buffy = new StringBuffer(400);
    buffy.append(JuiceUtil.arrayToString(digs,",") + " * " + base +"^" + exp);
    float dd = digits(digs);
    buffy.append(" = " + dd + " * b^" + exp);
    float ff = valueOf(dd); 
    buffy.append(" = " + ff);
    System.out.println(new String(buffy)); 
  }

  void setRandomDigitals() {
    Random rand = new Random(1860); 
    for (int i = 0; i < digs.length; i++) {
      float dummy = rand.nextFloat(); 
      if (dummy != 1f) digs[i] = (int) (dummy * base);
    } 
  }
}



