package cohy.datstat; 

import juice.util.Tester; 

public class Recursion {
  public static void main(String[] args) {
    boolean correct = true; 
    int intResult; 

    intResult = Stirling.nextRecursion(4,2); 
    System.out.println("Stirling of 4 over 2 should be 7: " + intResult);
    if (intResult != 7) correct = false; 

    intResult = Factorial.nextRecursion(3); 
    System.out.println("Factorial of 3 should be 6:" + intResult);
    if (intResult != 6) correct = false;

    int[] array = {1,2,3,4,6,7,8,9};
    intResult = Sum.nextRecursion(array); 
    System.out.println("Sum should be 40:" + intResult);
    if (intResult != 40) correct = false; 

    float[] farray = {1.4f,2.2f,3.6f,4.2f,6.4f,7.1f,8.f,9.2f};
    intResult = BinarySearch.firstRecursion(farray,2.2f); 
    System.out.println("BinarySearch should be 1:" + intResult);
    if (intResult != 1) correct = false; 

    float floatResult = Power.nextRecursion(3.f, 7); 
    System.out.println("3^7 should be 2187:" + floatResult);
    if (floatResult != 2187.f) correct = false; 

    Tester.exit(correct); 
  }
}

/** 
 *   n
 * [   ] Stirling number 
 *   k
 */ 
class Stirling {
  static int nextRecursion(int nn, int kk) {
    if (nn < kk)            return 0; 
    if (nn >  0 && kk == 0) return 0;
    if (nn == kk)           return 1;
    return kk * Stirling.nextRecursion(nn-1, kk   ) + 
                Stirling.nextRecursion(nn-1, kk -1);
  }
}

/**
 * n! Factorial n * (n-1) * (n-2) * ... * 2 * 1
 */
class Factorial {
  static int nextRecursion(int number) {
    if (number == 0) return 1; 
    return (number * Factorial.nextRecursion(number -1));
  }
}

/** 
 * Sum x_i = x_0 + x_1 + ... + x_n
 */
class Sum {
  static int nextRecursion(int[] array) {
    return nextRecursion(array, 0); 
  }
  static int nextRecursion(int[] array, int pos) {
    if (pos+1 == array.length) return array[pos]; 
    return ( array[pos] + Sum.nextRecursion(array, pos+1) );
  }
}

/** 
 * recursive binary search algorithm: assumes arr is sorted in ascending order.
 * (Flamig, Practical Algorithms, pg 35)
 */
class BinarySearch {
  static int firstRecursion(float[] arr, float x) {
    return nextRecursion(arr, x, 0, arr.length);
  }
  static int nextRecursion(float[] arr, float x, int btm, int top) {
    if (top >= btm) {
      int mid = (top + btm) /2; 
      if (arr[mid] == x) return mid; // Return matching index.
      if (arr[mid] <  x) btm = mid+1; 
      else               top = mid-1;
      return BinarySearch.nextRecursion(arr, x, btm, top); 
    } 
    else return -1; // Number not found. 
  }
}

/** 
 * Recursive power function (Flamig, Practical Algorithms, pg 35)
 * This is a n log(n) algorithm. 
 */ 
class Power { 
  static float nextRecursion(float x, int n) {
    if (n == 0) return 1; 
    if (n == 1) return x; 
    if ((n % 2) != 0) return Power.nextRecursion(x, n-1) * x;  // for odd  n
    else        return Power.nextRecursion(x*x, n/2)  ;  // for even n
  }
}

