package   cohy.estDip;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   conv.convi.*;
import   sep.io.ArgumentKeeper;

/**
 * Estimates the plane-wave component in a data. 
 * Returns dip as an Rsf (number of elements = dimension of input). 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class DipEstimator extends SepOperator{
  PartDiffDots pdif; 
  Vector       dots;

  /** indices that identify slopes in output array.*/
  public final static int DX = 0; 
  public final static int DY = 1;
  public final static int DZ = 2;   

  public DipEstimator(RsfSpace iputSpace) {
    this(iputSpace, DipEstimator.getOutputRsfSpace(iputSpace));
  }
  public DipEstimator(RsfSpace iputSpace, RsfSpace oputSpace) {
    super(iputSpace, oputSpace);
    int dim   = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
    this.pdif = new PartDiffDots(iputSpace);
    this.dots = pdif.getRange().newMember();
  }
  protected DipEstimator(DipEstimator src) {
    super(src);
    this.pdif = new PartDiffDots((RsfSpace) src.getDomain()); 
    this.dots = pdif.getRange().newMember(); 
  }
  public jam.JamObject copy() { return new DipEstimator(this);}
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    return new RsfSpace(iputSpace.getDimension());
  }
  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  = ((Rsf) iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    pdif.image(iput, dots); 
    int[] ixy = {PartDiffDots.XY2}; float xy = ((Rsf)dots).getDatum(ixy);
    int[] iyy = {PartDiffDots.YY2}; float yy = ((Rsf)dots).getDatum(iyy);
    int[] dx = {DX}; ((Rsf)oput).setDatum(dx, -xy/yy); 
    int[] dy = {DY}; ((Rsf)oput).setDatum(dy,    1.f);
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    pdif.image(iput, dots);
    int[] ixz = {PartDiffDots.XZ3}; float xz = ((Rsf)dots).getDatum(ixz);
    int[] iyz = {PartDiffDots.YZ3}; float yz = ((Rsf)dots).getDatum(iyz);
    int[] izz = {PartDiffDots.ZZ3}; float zz = ((Rsf)dots).getDatum(izz);

//System.err.println("zz  = " + zz);
    float qx  = xz / zz;
    float qy  = yz / zz;
    float det = qx*qx + qy*qy + 1;

    float dipx = (1.f/det) * ((qx*qx + 1) * qx + (qy*qx    ) * qy);
    float dipy = (1.f/det) * ((qx*qy    ) * qx + (qy*qy + 1) * qy);

//System.err.println("det  = " + det);
//System.err.println("dipx = " + dipx);
//System.err.println("dipy = " + dipy);

    int[] dx = {DX}; ((Rsf)oput).setDatum(dx, dipx); 
    int[] dy = {DY}; ((Rsf)oput).setDatum(dy, dipy);
    int[] dz = {DZ}; ((Rsf)oput).setDatum(dz,  1.f); 
  }
}


