package   cohy.estDip;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   cohy.gradient.*;
import   conv.convi.*;
import   sep.io.ArgumentKeeper;

/**
 * Estimates the dip of the input data and fills an equally sized output
 * data file. If the input is a 3D volume, the output is filled with the 
 * norm of the dip vector. Uses DipEstimator to compute the dip. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class DipFiller extends SepOperator{
  DipEstimator dipy; // could create on the fly: clutter vs efficiency
  Vector       dips; // could create on the fly: clutter vs efficient?

  public DipFiller(RsfSpace iputSpace) {
    this(iputSpace, DipFiller.getOutputRsfSpace(iputSpace));
  }
  public DipFiller(RsfSpace iputSpace, RsfSpace oputSpace) {
    super(iputSpace, oputSpace);
    int dim   = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
    this.dipy = new DipEstimator(iputSpace);
    this.dips = dipy.getRange().newMember();
  }
  protected DipFiller(DipFiller src) {
    super(src);
    dipy = new DipEstimator((RsfSpace) src.getDomain()); 
    dips = dipy.getRange().newMember(); // should this copy the values? 
  }
  public jam.JamObject copy() { return new DipFiller(this);}

  /**
   * This should be using the IcaiFactory class once that class implements
   * the OperatorFactory interface. Until then this class is a hack. 
   * I should probably call this createRange(). 
   */
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    return new RsfSpace(iputSpace.getAxes()); 
  }
  /**
   * I should call this createDomain(). More comments see getOutputRsfSpace.
   */
  public static RsfSpace getInputRsfSpace(RsfSpace oputSpace) {
    return new RsfSpace(oputSpace.getAxes()); 
  }

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  = ((Rsf) iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);
System.err.println("dx : " + dx + " dy : " + dy);
    ((Rsf) oput).fill(-dx/dy);
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);
    int[] idz = {DipEstimator.DZ}; float dz = ((Rsf)dips).getDatum(idz);
    ((Rsf) oput).fill((float) Math.sqrt(dx*dx + dy*dy + dz*dz));
  }
}
