package   cohy.estDip;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   cohy.gradient.*;
import   conv.convi.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the compound of the crossproduct 
 * <tt>({\bf p \times \nabla) f</tt> and its adjoint. 
 * If <tt>f<tt> is a plane-wave and <tt>p</tt> its dip, the forward operator 
 * will return a null vector.<BR> 
 *
 * The output is always equal to the input vector. 
 * 
 * I should write DipXGrad as linear operator with Adj. This would be 
 * simply the compound of the two. 
 *  
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class DipXGradAndAdj extends SepOperator{
  DipEstimator dipy; // could create on the fly: clutter vs efficiency
  Vector       dips; // could create on the fly: clutter vs efficient?

  public DipXGradAndAdj(RsfSpace iputSpace) {
    this(iputSpace, DipXGradAndAdj.getOutputRsfSpace(iputSpace));
  }
  public DipXGradAndAdj(RsfSpace iputSpace, RsfSpace oputSpace) {
    super(iputSpace, oputSpace);
    int dim   = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
    this.dipy = new DipEstimator(iputSpace);
    this.dips = dipy.getRange().newMember();
  }
  protected DipXGradAndAdj(DipXGradAndAdj src) {
    super(src);
    dipy = new DipEstimator((RsfSpace) src.getDomain()); 
    dips = dipy.getRange().newMember(); // should this copy the values? 
  }
  public jam.JamObject copy() { return new DipXGradAndAdj(this);}

  /**
   * This should be using the IcaiFactory class once that class implements
   * the OperatorFactory interface. Until then this class is a hack. 
   * I should probably call this createRange(). 
   */
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    return new RsfSpace(iputSpace); 
  }
  /**
   * I should call this createDomain(). More comments see getOutputRsfSpace.
   */
  public static RsfSpace getInputRsfSpace(RsfSpace oputSpace) {
    return new RsfSpace(oputSpace); 
  }

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  = ((Rsf) iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);


    // create the convolution operator. 
    Rsf fd   = PartDiff2d.getDerivative(dy, -dx);
    IcaiFactory fac = new IcaiFactory(fd);
    Icai diff = fac.getIcaiFromInputSpace((RsfSpace) iput.getSpace());
    Rsf temp = (Rsf) diff.getRange().newMember();
    diff.image(iput, temp);
    diff.adjoint().image(temp, oput); 
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);
    int[] idz = {DipEstimator.DZ}; float dz = ((Rsf)dips).getDatum(idz);

    // create the convolution operator. 

    Rsf fd; IcaiFactory fac; Icai diff; Rsf temp; 
    fd   = PartDiff3d.getDerivative(0.f, -dz,  dy);

    fac  = new IcaiFactory(fd);
    diff = fac.getIcaiFromInputSpace((RsfSpace) iput.getSpace());
    temp = (Rsf) diff.getRange().newMember();
    diff.image(iput, temp);
    diff.adjoint().addImage(temp, oput); 
      
    fd   = PartDiff3d.getDerivative( dz, 0.f, -dx);
    fac  = new IcaiFactory(fd);
    diff = fac.getIcaiFromInputSpace((RsfSpace) iput.getSpace());
    diff.image(iput, temp);
    diff.adjoint().addImage(temp, oput); 

    fd   = PartDiff3d.getDerivative(-dy,  dx, 0.f);
    fac  = new IcaiFactory(fd);
    diff = fac.getIcaiFromInputSpace((RsfSpace) iput.getSpace());
    diff.image(iput, temp);
    diff.adjoint().addImage(temp, oput); 
  }
}


