package   cohy.estDip;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   cohy.gradient.*;
import   conv.convi.*;
import   sep.io.ArgumentKeeper;

/**
 * Computes the norm of the crossproduct 
 * <tt>| ({\bf p \times \nabla) f |</tt>. The operator fills the output 
 * vector with the norm result (scalar).  
 * If <tt>f<tt> is a plane-wave and <tt>p</tt> its dip, the operator 
 * will return zero.<BR> 
 * In 2D the operator is simply the norm of the partial differential operator 
 * <tt>| (p_y \partial_x + p_x \partial_y) f |</tt>.
 * 
 * Maybe a better design to hand in the dip and to make this the linear 
 * part of a SemiLinear operator. I also should pack the 6 dimensional 
 * output into a single vector. But I don't think the result will warrant 
 * the effort. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class DipXGradNorm extends SepOperator{
  DipEstimator dipy; // could create on the fly: clutter vs efficiency
  Vector       dips; // could create on the fly: clutter vs efficient?

  public DipXGradNorm(RsfSpace iputSpace) {
    this(iputSpace, DipXGradNorm.getOutputRsfSpace(iputSpace));
  }
  public DipXGradNorm(RsfSpace iputSpace, RsfSpace oputSpace) {
    super(iputSpace, oputSpace);
    int dim   = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
    this.dipy = new DipEstimator(iputSpace);
    this.dips = dipy.getRange().newMember();
  }
  protected DipXGradNorm(DipXGradNorm src) {
    super(src);
    dipy = new DipEstimator((RsfSpace) src.getDomain()); 
    dips = dipy.getRange().newMember(); // should this copy the values? 
  }
  public jam.JamObject copy() { return new DipXGradNorm(this);}

  /**
   * This should be using the IcaiFactory class once that class implements
   * the OperatorFactory interface. Until then this class is a hack. 
   * I should probably call this createRange(). 
   */
  public static RsfSpace getOutputRsfSpace(RsfSpace iputSpace) {
    int n = iputSpace.getDimension();
    Axis[] oputAxes = new Axis[n];
    for (int i = 0 ; i < n ; i++) {
      oputAxes[i]          = new Axis(iputSpace.getAxes()[i]);
      oputAxes[i].nsamples =          iputSpace.getNSamples(i) -1;
    }
    return new RsfSpace(oputAxes); 
  }
  /**
   * I should call this createDomain(). More comments see getOutputRsfSpace.
   */
  public static RsfSpace getInputRsfSpace(RsfSpace oputSpace) {
    int n = oputSpace.getDimension();
    Axis[] iputAxes = new Axis[n];
    for (int i = 0 ; i < n ; i++) {
      iputAxes[i]          = new Axis(oputSpace.getAxes()[i]);
      iputAxes[i].nsamples =          oputSpace.getNSamples(i) +1;
    }
    return new RsfSpace(iputAxes); 
  }

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  = ((Rsf) iput).getDimension();
    switch (dim) {
    case 2: apply2d(add, iput, oput); break;
    case 3: apply3d(add, iput, oput); break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
      break;
    }
  }

  void apply2d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);

System.err.println("dx : " + dx + "  dy : " + dy);

    // create the convolution operator. 
    Rsf ff = PartDiff2d.getDerivativeDx();
    InternalConvolution diff = 
      new InternalConvolution( InternalConvolution.kInput , 
			       (RsfSpace) iput.getSpace(),
			       InternalConvolution.kFilter,
			       (RsfSpace) ff.getSpace());

    //Rsf fd   = PartDiff2d.getDerivative(1.f, -dx/dy);
    Rsf fd   = PartDiff2d.getDerivative(dy, -dx);
    // apply the differential 
    diff.imageBinary(iput, fd, oput);
    ((Rsf) oput).fill(oput.norm());
  }

  void apply3d(boolean add, Vector iput, Vector oput) {
    dipy.image(iput, dips); 
    int[] idx = {DipEstimator.DX}; float dx = ((Rsf)dips).getDatum(idx);
    int[] idy = {DipEstimator.DY}; float dy = ((Rsf)dips).getDatum(idy);
    int[] idz = {DipEstimator.DZ}; float dz = ((Rsf)dips).getDatum(idz);

    // create the convolution operator. 
    Rsf ff = PartDiff3d.getDerivativeDx();
    InternalConvolution diff = 
      new InternalConvolution( InternalConvolution.kInput , 
			       (RsfSpace) iput.getSpace(),
			       InternalConvolution.kFilter,
			       (RsfSpace) ff.getSpace());

    Rsf fd; 
    fd   = PartDiff3d.getDerivative(0.f, -dz,  dy);
    diff.imageBinary(iput, fd, oput);
    float xc = oput.norm2();
System.err.println("xc is 0: " + xc);
    fd   = PartDiff3d.getDerivative( dz, 0.f, -dx);
    diff.imageBinary(iput, fd, oput);
    float yc = oput.norm2();
System.err.println("yc is 0: " + yc);
    fd   = PartDiff3d.getDerivative(-dy,  dx, 0.f);
    diff.imageBinary(iput, fd, oput);
    float zc = oput.norm2();
System.err.println("zc is 0: " + zc);
    ((Rsf) oput).fill((float) Math.sqrt(xc + yc + zc));
System.err.println("sum is 0: " + Math.sqrt(xc + yc + zc));
  }
}


