package   cohy.estWave;

import 	  sep.operator.*;
import  juice.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import   cohy.estDip.*;
import    rsf.operator.*;

/**
 * Given an input Rsf, this operator 
 * estimates the least-square plane-wave dip of the input, 
 * stacks the input perpendicular to the dip direction to a single trace, 
 * spreads the single trace to a vector of the same space as the input: 
 * the best fitting plane-wave component of the input vector.  
 *  
 * This could be a semilinear operator. Need to think about this a bit more.
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class ProPro extends SepOperator{
  DipEstimator dipy; 
  Rsf1         dips; 
  Projection   proj;

  public ProPro(RsfSpace iputSpace) {
    super(iputSpace, iputSpace);
    int dim   = iputSpace.getDimension();
    if (dim < 2 || dim > 3) System.err.println("Error: not implemented ..");
    this.dipy = new DipEstimator(iputSpace);
    this.dips = (Rsf1) dipy.getRange().newMember();
    this.proj = new Projection(iputSpace);
  }
  protected ProPro(ProPro src) {
    super(src);
    dipy = (DipEstimator) src.dipy.copy();
    dips = (Rsf1)         src.dips.copy();
    proj = (Projection)   src.proj.copy();
  }
  public jam.JamObject copy() { return new ProPro(this);}

  protected void apply(boolean add, Vector iput, Vector oput) {
    int dim  = ((Rsf) iput).getDimension();
    dipy.image(iput, dips);
    // next not necessary: DipEstimator & Projection need common dip format. 
    float[] steps = null; 
    switch (dim) { 
    case 2:
      steps = new float[2];
      steps[0] = dips.getDatum(DipEstimator.DX);
      steps[1] = dips.getDatum(DipEstimator.DY);
      break; 
    case 3:
      steps = new float[3];
      steps[0] = dips.getDatum(DipEstimator.DX);
      steps[1] = dips.getDatum(DipEstimator.DY);
      steps[2] = dips.getDatum(DipEstimator.DZ);
      break;
    default:
      System.err.println("Error: dimensions not implemented yet."); 
    }
    proj.setStepout(steps); 
    proj.setWeights(); 
    CompoundLinOpAdj projProj = new CompoundLinOpAdj(proj, proj.adjoint());
    projProj.image(iput,oput); 
  }
}


