package   cohy.gradient;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import    sep.io.*;

/**
 * Implements a simple 4-point difference star with adjustable coefficients.
 * <BR>
 * output = (dx Dx + dt Dt) input (Dx and Dt are partial differential ops).
 *
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class PartDiff2d {

  private static Rsf2 newRsf() {
    int[] sz = {2,2};
    return (Rsf2) RsfFactory.newRsf(sz); 
  }

  /**
   * @param dx=1 dt=0 yields (Dx input)
   * @param dx=0 dt=1 yields (Dt input)
   * @param dx=1 dt=1 yields (Dx + Dt) input (default)
   */
  public static Rsf getDerivative(float dx, float dt) { 
    Rsf2 rsf = newRsf(); float[][] star = rsf.d;
    star[0][0] = - dx - dt; star[1][0] =   dx - dt;
    star[0][1] = - dx + dt; star[1][1] =   dx + dt;
    return rsf;
  }
  public static Rsf getDerivativeDx() { 
    Rsf2 rsf = newRsf(); float[][] star = rsf.d;
    star[0][0] = - 1.f; star[1][0] =   1.f;
    star[0][1] = - 1.f; star[1][1] =   1.f;
    return rsf;
  }
  public static Rsf getDerivativeDt() { 
    Rsf2 rsf = newRsf(); float[][] star = rsf.d;
    star[0][0] = - 1.f; star[1][0] = - 1.f;
    star[0][1] =   1.f; star[1][1] =   1.f;
    return rsf;
  }

  /**
   * PartDiff dx=1.f dt=1.f > rsf.H
   * @param dx=1 dt=0 yields Dx = ((-1,-1),( 1,1)) 
   * @param dx=0 dt=1 yields Dt = ((-1, 1),(-1,1)) 
   * @param dx=1 dt=1 yileds (Dx + Dt) = ((-2, 0),( 0, 2)) 
   */
  public static void main(String[] args) {
    ArgumentKeeper command = new ArgumentKeeper(args);

    float dx = command.getFloat("dx", 1.f); 
    float dt = command.getFloat("dt", 1.f); 

    Rsf oput = PartDiff2d.getDerivative(dx,dt);
    oput.write(System.out); 
  }
}

