package   cohy.gradient;

import 	  sep.operator.*;
import    jam.vector.*;
import 	  rsf.vector.*;
import    sep.io.*;

/**
 * Implements a simple 4-point difference star with adjustable coefficients.
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class PartDiff3d {

  private static Rsf3 newRsf() {
  int[] sz = {2,2,2};
  Rsf3 rsf = (Rsf3) RsfFactory.newRsf(sz); 
  return rsf;
  }

  /**
   * @param dx=1 dy=0 dz=0 yields (Dx input)
   * @param dx=0 dy=1 dz=0 yields (Dy input)
   * @param dx=1 dy=1 dz=1 yields (Dx + Dy + Dz) input 
   */
  public static Rsf getDerivative(float dx, float dy, float dz) { 
    Rsf3 rsf = newRsf(); float[][][] star = rsf.d;
    star[0][0][0] = - dx - dy - dz; star[1][0][0] =   dx - dy - dz;
    star[0][0][1] = - dx - dy + dz; star[1][0][1] =   dx - dy + dz;
    star[0][1][0] = - dx + dy - dz; star[1][1][0] =   dx + dy - dz;
    star[0][1][1] = - dx + dy + dz; star[1][1][1] =   dx + dy + dz;
    return rsf; 
  }
  public static Rsf getDerivativeDx() { 
    Rsf3 rsf = newRsf(); float[][][] star = rsf.d;
    star[0][0][0] = - 1.f; star[1][0][0] =   1.f;
    star[0][0][1] = - 1.f; star[1][0][1] =   1.f;
    star[0][1][0] = - 1.f; star[1][1][0] =   1.f;
    star[0][1][1] = - 1.f; star[1][1][1] =   1.f;
    return rsf;
  }
  public static Rsf getDerivativeDy() { 
    Rsf3 rsf = newRsf(); float[][][] star = rsf.d;
    star[0][0][0] = - 1.f; star[1][0][0] = - 1.f;
    star[0][0][1] = - 1.f; star[1][0][1] = - 1.f;
    star[0][1][0] =   1.f; star[1][1][0] =   1.f;
    star[0][1][1] =   1.f; star[1][1][1] =   1.f;
    return rsf;
  }
  public static Rsf getDerivativeDz() { 
    Rsf3 rsf = newRsf(); float[][][] star = rsf.d;
    star[0][0][0] = - 1.f; star[1][0][0] = - 1.f;
    star[0][0][1] =   1.f; star[1][0][1] =   1.f;
    star[0][1][0] = - 1.f; star[1][1][0] = - 1.f;
    star[0][1][1] =   1.f; star[1][1][1] =   1.f;
    return rsf;
  }

  public static void main(String[] args) {
    ArgumentKeeper command = new ArgumentKeeper(args);

    float dx = command.getFloat("dx", 1.f);
    float dy = command.getFloat("dy", 1.f);
    float dz = command.getFloat("dz", 1.f);

    PartDiff3d pdiff = new PartDiff3d();
    Rsf oput = pdiff.getDerivative(dx,dy,dz);
    oput.write(System.out); 
  }
}

