package cohy.patch3d;

import jam.vector.*;
import rsf.vector.*;
import rsf.solver.*;
import sep.io.ArgumentKeeper;
import juice.solver.*;
import conv.convi.*;
import nstat.patch.*;
import java.util.Enumeration;

public class NonStationaryPefFilter {
  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);

    // iquilt specs (nonstationary Input) //
    String dataDir = "../Data/";
    String  inRsf  = argKeeper.getString("in" , dataDir + "gulfAntiRaw.H");
    Rsf data       = RsfFactory.newRsf(inRsf);
    System.err.println(data);

    // ipatch specs //
    float  overlap = argKeeper.getFloat ("overlap", "0.5");
    int[] patchSize = { ((RsfSpace) data.getSpace()).getNSamples(0),
			10, 10 };  // full version
    // int[] patchSize = {  4, 10, 10 };  // windowed version
    System.err.println(patchSize);

    // filter specs //
    int[] filtSize    = {  3,  3,  2 };
    int[] filtOne     = {  1,  1,  0 };
    Rsf   pefKnown    = getPefKnown   (filtSize, filtOne);
    Mask  pefFreeMask = getPefFreeMask(filtSize, filtOne);
    Rsf   filter      = (Rsf) pefKnown.getSpace().newMember();
    System.err.println("filter: " + filter); 
    System.err.println("known : " + pefKnown); 
    System.err.println("mask  : " + pefFreeMask); 

    // deconvolution specs //
    float  tol     = argKeeper.getFloat ("tol"    , "1e-10");
    // set # iterations to roughly 2 * (# unkowns) for CG
    int    niter   = argKeeper.getInt   ("niter"  , "20");
    niter          = 2 * filtSize[0]*filtSize[1]*filtSize[2];

    // oquilt comps // 
    InternalConvolution
	oConv = new InternalConvolution(InternalConvolution.kInput ,
				       (RsfSpace) data.getSpace(),
				       InternalConvolution.kFilter,
				       (RsfSpace) filter.getSpace());
    Rsf out = (Rsf) oConv.getRange().newMember(); 
    out.zero();         // this may be redundant in Java
    System.err.println(out);

    Rsf quiltWeight = (Rsf) out.getSpace().newMember();
    quiltWeight.zero();

    PatchFactory pf = new PatchFactory((RsfSpace) data.getSpace(),
				 patchSize,
				 overlap);
    Patch pw = pf.getPatch();

    int numPatches = pw.size();
    int i = 1;
    for (Enumeration e = pw.getWindows(); e.hasMoreElements(); i++) {
      System.out.println("Doing patch #" + i + " of " + numPatches);

      Window cut = (Window) e.nextElement();
      Vector dataPatch = cut.image(data);

      filter.zero();

      MissingFilter.getMissingFilter((Rsf) dataPatch, pefKnown,
				     pefFreeMask, filter, tol, niter);
      filter.add(pefKnown);

      InternalConvolution
	conv = new InternalConvolution(InternalConvolution.kInput ,
				       (RsfSpace) dataPatch.getSpace(),
				       InternalConvolution.kFilter,
				       (RsfSpace) filter   .getSpace());
      Vector filtered = conv.imageBinary(cut.image(data), filter);

      Window pad = new Window( (RsfSpace) out.getSpace(), 
                               (RsfSpace) filtered.getSpace(), 
                               cut.getAnchor(), 
			       false); 
      pad.image(filtered,out); 
    }

    filter.write("filter.H");
    String outRsf  = argKeeper.getString("out", dataDir + "NSPEFout.H");
    out.write(outRsf);
  }

  static Rsf  getPefKnown   (int[] pefSize, int[] filtOne) {
    Rsf pef = RsfFactory.newRsf(pefSize);
    pef.zero();
    pef.setDatum(filtOne, 1F);
    return pef;
  }

  static Mask getPefFreeMask(int[] pefSize, int[] filtOne) {
    Rsf mask = RsfFactory.newRsf(pefSize);
    mask.fill(1F);
    for (int i2 = 0; i2 <= filtOne[2]; i2++) 
    for (int i1 = 0; i1 <= filtOne[1]; i1++) 
    for (int i0 = 0; i0 <= filtOne[0]; i0++) 
      ((Rsf3)mask).d[i2][i1][i0] = 0F; 
    return new SepMask(mask);
  }
}
