package  cohy.patch3dF;

import   rsfF.vector.*;
import   rsfF.solver.*;
import   conv.conviF.*;
import  juice.solver.*;

/**
 * Use <TT>MissingFilter.getMissingFilter()</TT> to calculate a filter
 * for data, where data has no gaps.  That is, only the filter is unknown,
 * and the data is completely known.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class MissingFilter {
  public static void getMissingFilter(Rsf data,
				      Rsf filtKnown, Mask filtFreeMask,
				      Rsf result,
				      float tol, int niter) {
    Icaf icaf =
      new InternalConvolution(InternalConvolution.kInput,
			      (RsfSpace) data.getSpace(),
			      InternalConvolution.kFilter,
			      (RsfSpace) filtKnown.getSpace()).getIcaf(data);

    MaskedEqns mdeq  = new MaskedEqns(icaf, filtKnown, filtFreeMask);
    mdeq.zeroKnownComponents(result);
    NormalEqns nreq  = new NormalEqns(mdeq.getAM(), mdeq.getNegAk());
    CGSolver        cgSlv = new CGSolver(new IteratorSimple(tol, niter, true));
      
    cgSlv.solve(nreq.getAtA(), nreq.getAtb(), result);
  }
}
