package geom.proj;

import rsf.vector.*;
import sep.io.*;

/**
 * Creates a synthetic folt image. Compare with geom.simple.SynFolt. 
 * This version uses Projection. It has too many options. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class SynFolt {

  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args); 
    int[] n = cmd.getIntArray("n","100,50,10"); 
    RsfSpace space = new RsfSpace(n); 

    // nf^t x = thck
    int thck = cmd.getInt("thck","5");
    float[] nf = cmd.getFloatArray("nf","-2.f,3.f,1.f"); 
    float norm = 0.f; 
    for (int i=0; i<nf.length; i++)  norm += nf[i]*nf[i]; 
    norm = (float) Math.sqrt(norm);
    for (int i=0; i<nf.length; i++) 
      nf[i] /= norm; 

    PlaneWave generator = new PlaneWave(space);
    generator.setTraceFilter(rsf.util.Tent.newRsf(7)); 
    Rsf folt = (Rsf) space.newMember(); float[][][] ff = ((Rsf3) folt).d; 
    generator.setStepout(nf);
    generator.addImage(folt,folt);


    Rsf left = (Rsf) space.newMember(); float[][][] ll = ((Rsf3) left).d; 
    if (cmd.containsKey("nlc")) {                         // fill with constant
      left.fill(cmd.getFloat("nlc"));
    } 
    else {
      float[] nl = cmd.getFloatArray("nl","0.f,0.f,1.f"); // fill with wave
      generator.setStepout(nl);
      generator.addImage(left,left);
    }

    Rsf rght = (Rsf) space.newMember(); float[][][] rr = ((Rsf3) rght).d; 
    if (cmd.containsKey("nrc")) {                         // fill with constant
      left.fill(cmd.getFloat("nrc"));
    } 
    else {
      float[] nr = cmd.getFloatArray("nr","1.f,.5f,.9f"); // fill with wave
      generator.setStepout(nr);
      generator.addImage(rght,rght);
    }

    Rsf data = (Rsf) space.newMember(); float[][][] dd = ((Rsf3) data).d; 
    if ( nf[0] == 0.f ) System.err.println("Error: no vertical line!");
    if ( nf[0]  < 0.f ) System.err.println("Error: only ascending fault!");

    float h2 = n[2]/2.f; 
    float h1 = n[1]/2.f; 
    float h0 = n[0]/2.f; 
    for     (int i2=0; i2<n[2]; i2++) {  
      for   (int i1=0; i1<n[1]; i1++) {
	// nf^t (x1 + n/2) =   thck        // RSF INDX SHT
	// nf^t (x2 + n/2) = - thck        // RSF INDX SHT
	int z1 = (int) (( -nf[0]*(i2-h2) - nf[1]*(i1-h1) - thck) / nf[2] + h0);
	int z2 = (int) (( -nf[0]*(i2-h2) - nf[1]*(i1-h1) + thck) / nf[2] + h0);
	for (int i0=0; i0<n[0]; i0++) { 
	  if (i0 >= z1            ) dd[i2][i1][i0] = ll[i2][i1][i0]; 
	  if (i0 <  z1 && i0 >  z2) dd[i2][i1][i0] = ff[i2][i1][i0]; 
	  if (            i0 <= z2) dd[i2][i1][i0] = rr[i2][i1][i0]; 
    } } }
    data.setTitle(cmd.getString("rsfName","Synthetics")) ; 
    data.write(System.out); 
  }
}

