package  geom.shift;

import  java.util.Random;
import   rsf.vector.*;
import   rsf.operator.*;
import   rsf.util.*;
import   sep.io.ArgumentKeeper;

/**
 * Creates a synthetic 2-D data set that simulates ascending and 
 * descending planar events (Similar to satelite tracks). 
 * The original program was written in Ratfor by Jon.<BR> 
 * @param n0=[80], n1=[80], n2=[80] are the number of samples for data Rsf.
 * @param s0=[1] , s1=[1] , s2=[1]  are the stepout in samples.
 * @param pow=[1]                   pow(d[i],pow) for all samples i of rand.
 * @param len=[22]                  half-length of the tent smooting filter. 
 * 
 * PlaneWaveVol > out.H 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class PlaneWaveVol {
  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    float pow = cmd.getFloat("pow",1.f);
    int   len = cmd.getInt  ("len",22);
    int n0 = cmd.getInt("n0", 80); int s0 = cmd.getInt("s0", 1);
    int n1 = cmd.getInt("n1", 80); int s1 = cmd.getInt("s1", 1);
    int n2 = cmd.getInt("n2", 80); int s2 = cmd.getInt("s2", 1);
    Rsf3 dipdat = (Rsf3) RsfFactory.newRsf(n0,n1,n2); 

    Rsf1 ss = (Rsf1) RsfFactory.newRsf(14*n0); 
    ss.rand(new Random(2001)); ElwPow.elwPow(ss,pow);
    ss = (Rsf1) Convolver.convolve(Tent.newRsf(len), ss);
    for (int i2 = 0; i2 < n2; i2++) {
    for (int i1 = 0; i1 < n1; i1++) {
    for (int i0 = 0; i0 < n0; i0++) {
      dipdat.d[i2][i1][i0]  = ss.d[s0 * i0 + 7*n0 + s1 * i1 + s2 * i2];
    } } }
    dipdat.setTitle("Plane_wave_Volume");
    dipdat.write(System.out);
  }
}

