package geom.sigmoid;

import java.util.Random; 
import rsf.vector.*; 
import sep.io.ArgumentKeeper;

/** 
 * This makes quarter dome model. 
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matthias Schwab (after Jon Claerbout)  */ 

public class Qdome { 
  /** 
   * Qdome [n0=400 n1=100 n2=50 o0=0 d0=.004 ... ] > out.H
   */ 
  public static void main(String[] args) { 
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);
    int    n0 = argKeeper.getInt  ("n0"  , "200" ); // was 400 // 
    float  d0 = argKeeper.getFloat("d0"  , ".008");
    float  o0 = argKeeper.getFloat("o0"  , "0.0" );
    int    n1 = argKeeper.getInt  ("n1"  , "50"  ); // was 100 // 
    float  d1 = argKeeper.getFloat("d1"  , ".01" );
    float  o1 = argKeeper.getFloat("o1"  , "0.0" );
    int    n2 = argKeeper.getInt  ("n2"  , "25"  ); // was 50  //
    float  d2 = argKeeper.getFloat("d2"  , ".02" );
    float  o2 = argKeeper.getFloat("o2"  , "0.0" );
    Axis[] ax = new Axis[3]; 
    ax[0] = new Axis(n0,d0,o0,"time","sec");
    ax[1] = new Axis(n1,d1,o1,"W-E","km" );
    ax[2] = new Axis(n2,d2,o2,"N-S","km" );
    Rsf modl = RsfFactory.newRsf(ax); 
    float ranget   = argKeeper.getFloat("ranget"  , "3"  );
    float gaussvel = argKeeper.getFloat("gaussvel", "2.5");
    float slip     = argKeeper.getFloat("slip"    , ".01");
    int   endtaper = argKeeper.getInt  ("endtaper", "20" );
    int f1 = (int) ((n1/2) -1); 
    if (argKeeper.containsKey("f1")) f1 = argKeeper.getInt  ("f1"); 
    int f2 = (int) ((n2/2) -1); 
    if (argKeeper.containsKey("f2")) f2 = argKeeper.getInt  ("f2"); 

    Qdome.createModel(modl,new Random(1815), 
		      ranget, gaussvel, slip, endtaper, f1,f2);
    modl.write(System.out);
  }
  public static void createModel(Rsf modl) { 
    Qdome.createModel(modl, new Random(1815), 
			3.f, 2.5f, .01f, 20, 
			(int) (modl.getNSamples()[1]/2 -1), 
			(int) (modl.getNSamples()[2]/2 -1)); 
  } 
  public static void createModel(Rsf modl, Random randy, 
				 float ranget, float gaussvel, float slip, 
				 int endtaper, int f1, int f2) {
    Axis[] ax = ((RsfSpace) modl.getSpace()).getAxes();

    int n0 = ax[0].nsamples; float d0 = ax[0].delta; float o0 = ax[0].offset;
    int n1 = ax[1].nsamples; float d1 = ax[1].delta; float o1 = ax[1].offset;
    int n2 = ax[2].nsamples; float d2 = ax[2].delta; float o2 = ax[2].offset;
    int large = (int) (ranget * n0); 

    float[] imp = new float[large];  // define: imp[i] + 1. = impedance > 0.f 
    for(int i=1; i < large; i++) { 
      if ( randy.nextFloat() > .3 ) { imp[i] = imp[i-1];               }
      else                          { imp[i] = .1f * randy.nextFloat();}
    }
    for (int i=1; i < large; i++)    // integrate the impedance  
      imp[i] = imp[i] + imp[i-1]; 

    float[][][] earth = new float[n2][n1][n0];
    for (int i2 = 0; i2 < n2; i2++) 
    for (int i1 = 0; i1 < n1; i1++) 
    for (int i0 = 0; i0 < n0; i0++) 
      earth[i2][i1][i0] = 1.f; 

    double thetac = 45. * 2. * Math.PI / 360.; 
    double dr     = Math.abs(d1) + Math.abs(d2); 
    double xmax   = o1 + d1 * (n1-1); 
    double ymax   = o2 + d2 * (n2-1);
    double rmax   = Math.sqrt(xmax*xmax + ymax*ymax) + dr; 
    double xp     = 0.75 * xmax; 
    double yp     = 0.75 * ymax; 
    double rcap   = 0.3f  * rmax; // rcap is the radius of the salt sphere //
    //double rad1   = Math.sqrt(xp*xp + yp*yp) - rcap; 
    double rad2   = Math.sqrt(xp*xp + yp*yp);
    int    n0a    = (int) (.15 * n0); 
    int    n0b    = (int) (.8  * n0); 
    double dz     = 1. / (n0b - n0a); 




    for (int i2 = 0; i2 < n2; i2++) { double y = (double) (o2 + d2 * i2);  
    for (int i1 = 0; i1 < n1; i1++) { double x = (double) (o1 + d1 * i1); 
      double r     = Math.sqrt(x*x + y*y) + dr/2.;

      for (int i0 = 0; i0 < n0a; i0++) {               // shallow beds //
	earth[i2][i1][i0] = 1.f + (imp[i0+1] - imp[i0]); 
      }
      for (int i0 = n0b+1; i0 < n0; i0++) {	            // deep beds //
	double t    = i0 + .3 * i1 + .2 * i2;
	int    it1  = (int) t;
	int    it2  = (int) t+1;
        double frac = t - it1;
	float  ival = (float) ((imp[it2] * (1.f-frac) + imp[it2+1] * frac) -
		               (imp[it1] * (1.f-frac) + imp[it1+1] * frac));
	earth[i2][i1][i0] =  1.f + ival;
      }
      for (int i0 = n0a; i0 < n0b+1; i0++) { 
	double z    = ((double) (i0 - n0a))/ (n0b - n0a);         // 0 to 1 //
	double zcap = z * gaussvel /4.0;
	//     z    = 1 - z;
	       z    = dz + (1. - 2.*dz) * z;		       // dz to 1-dz //
	double t1   = (large/gaussvel) *(r/rmax) *Math.sqrt(-Math.log(z+dz/2));
	double t2   = (large/gaussvel) *(r/rmax) *Math.sqrt(-Math.log(z-dz/2));

	if(t1 > t2) System.err.println("misconception");

	// faulting (needs rework: clearer geometric objects) // 
	double xnew =  Math.cos(thetac) * (x-xp) + Math.sin(thetac) * (y-yp);
	       xnew =  xnew * 0.75;
	double ynew = -Math.sin(thetac) * (x-xp) + Math.cos(thetac) * (y-yp);
	if ((rcap*rcap) > (xnew*xnew + ynew*ynew + zcap*zcap)) {
	  double zrad = (rad2 - r)/rcap;
	  if (r > rad2)  zrad = 0.0;   
	  t1 += slip * n0 * zrad; 
	  t2 += slip * n0 * zrad; 
	}

	int it1  = (int) t1;
	int it2  = (int) t2;
	if( it1 >= 0 && it1+2 < large) { double frac1 = t1 - it1; 
	if( it2 >= 0 && it2+2 < large) { double frac2 = t2 - it2;
	  double ival = ((imp[it2] * (1.-frac2) + imp[it2+1] * frac2) -
			 (imp[it1] * (1.-frac1) + imp[it1+1] * frac1));
	  earth[i2][i1][i0] = (float) (1. + ival / (t2 - t1));
	}}
      }
    }}

    for (int i2 = 0; i2 < n2; i2++) { // Form reflectivity from impedance //
    for (int i1 = 0; i1 < n1; i1++) { 
	float[] refl = new float[n0];
    for (int i0 = 1; i0 < n0; i0++) { 
      refl[i0] = ( earth[i2][i1][i0-1] - earth[i2][i1][i0]) / 
	         ( earth[i2][i1][i0-1] + earth[i2][i1][i0]);
    }
    for (int i0 = 0; i0 < n0; i0++) 
      earth[i2][i1][i0] = refl[i0];
    }}

    for (int i2 = 0; i2 < n2; i2++)  
    for (int i1 = 0; i1 < n1; i1++)  
    for (int i0 = 0; i0 < n0; i0++) 
      if (earth[i2][i1][i0] == 0.f) earth[i2][i1][i0] = Float.MIN_VALUE; 

/*
# write out trace set at well location
call srite('trace',earth(1,f2+1,f3+1),4*n1)
call srite('trace',earth(1,f2+2,f3+1),4*n1)
call srite('trace',earth(1,f2+1,f3+2),4*n1)
call srite('trace',earth(1,f2+2,f3+2),4*n1)

# temporal smoothing
do i3= 1, n3   {
do i2= 1, n2   {
	call copy( n1, earth(1,i2,i3), refl )
	call triangle( 2, 1, n1, refl, refl)
	call triangle( 3, 1, n1, refl, refl)
	call triangle( 4, 1, n1, refl, refl)
	        earth( 1,i2,i3) = 0.
	do i1=2,n1
		earth(i1,i2,i3) = refl(i1) - refl(i1-1)
	}}
*/

    for (int i2 = 0; i2 < n2; i2++) {            // top and bottom taper //
    for (int i1 = 0; i1 < n1; i1++) {
      for (int i0=0;    i0<endtaper;      i0++)  
	earth[i2][i1][i0] = earth[i2][i1][i0] * (i0+1)  / ((float) endtaper);
      for (int i0=n0-1; i0<n0-endtaper-1; i0--)  
	earth[i2][i1][i0] = earth[i2][i1][i0] * (n0-i0) / ((float) endtaper);
    }}

    ((Rsf3) modl).d = earth; 
  }
}

