package geom.sigmoid;

import java.util.Random; 
import rsf.vector.*; 
import sep.io.ArgumentKeeper;

/**
 * This makes sigmoid reflectivity model.
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 * @author: Matthias Schwab (after Jon Claerbout) 
 */ 

public class Sigmoid {
  /**
   * Sigmoid [n0=400 n1=100 o0=0 d0=.004 o1=0 d1=.032 ] > out.H
   */ 
  public static void main(String[] args) { 
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);
    // use the getAxis() element instead?
    int   n0 = argKeeper.getInt  ("n0"  , "400" );
    float d0 = argKeeper.getFloat("d0"  , ".004");
    float o0 = argKeeper.getFloat("o0"  , "0.0" );
    int   n1 = argKeeper.getInt  ("n1"  , "100" );
    float d1 = argKeeper.getFloat("d1"  , ".032");
    float o1 = argKeeper.getFloat("o1"  , "0.0" );
    Axis[] ax = new Axis[2]; 
    ax[0] = new Axis(n0,d0,o0,"time","sec");
    ax[1] = new Axis(n1,d1,o1,"dist","km" );
    Rsf modl = RsfFactory.newRsf(ax); 
    Sigmoid.createModel(modl);
    modl.write(System.out);
  }
  public static void createModel(Rsf modl) { 
    Sigmoid.createModel(modl, new Random(1815)); 
  } 
  public static void createModel(Rsf modl, Random randy) {
    int n0 = modl.getNSamples()[0]; 
    int n1 = modl.getNSamples()[1]; 
    int large = 5 * n0; 
    float[] imp1 = new float[large]; imp1[0] = 1.f; 
    for (int i=1; i< large; i++) {
      if ( randy.nextFloat() > .2 ) { imp1[i] = imp1[i-1];                   }
      else                          { imp1[i] = 1f + .1f * randy.nextFloat();}
    }
    float[] imp2 = new float[large]; imp2[0] = 1.f; 
    for (int i=1; i< large; i++) {
      if ( randy.nextFloat() > .3 ) { imp2[i] = imp2[i-1];                    }
      else                          { imp2[i] = 1f + .1f * randy.nextFloat();}
    }
    float[][] dipper = new float[n1][n0];
    for (int i1 = 0; i1 < n1; i1++) {
    for (int i0 = 0; i0 < n0; i0++) {
      float t    = i0 + i1 * .3f * (1.f * n0)/(1.f * n1);
      int   it   = (int) t; 
      float frac = t - it;
      dipper[i1][i0] = imp1[it] * (1.f -frac) + imp1[it+1] * frac; 
    }}
    float[][] sig1 = new float[n1][n0]; 
    float[][] sig2 = new float[n1][n0];
    for (int i1 = 0; i1 < n1; i1++) {
    for (int i0 = 0; i0 < n0; i0++) {
      float t    = i0 + n0 * .15f *(1.1f + (float)Math.cos((3.*3.14*i1)/n1));
      int   it   = (int) t; 
      int   is   = it + 10; 
      float frac = t - it ;
      sig1[i1][i0] = imp2[it] * (1.f - frac) + imp2[it+1] * frac; 
      sig2[i1][i0] = imp2[is] * (1.f - frac) + imp2[is+1] * frac;
    }}
    float[][] fault = new float[n1][n0];
    for (int i1 = 0; i1 < n1; i1++) {                  // fault // 
    for (int i0 = 0; i0 < n0; i0++) {   
      if ((((float) i1) / n1 - .9) < -.75 * Math.pow((i0 + 1.)/n0,2)) 
	fault[i1][i0] = sig1[i1][i0];
      else 
	fault[i1][i0] = sig2[i1][i0];
    }}
    float[][] earth = new float[n1][n0];
    for (int i1 = 0; i1 < n1; i1++) {
    for (int i0 = 0; i0 < n0; i0++) {
      int it = (int) (i0 + i1 * .3 * (1.*n0)/(1.*n1));
      if (it > n1/2) { earth[i1][i0] = fault [i1][i0]; }
      else           { earth[i1][i0] = dipper[i1][i0]; }
    }}
    float[][] refl = new float[n1][n0];     // impedance to reflectivity // 
    for (int i1 = 0; i1 < n1; i1++) {
    for (int i0 = 1; i0 < n0; i0++) {
      refl[i1][i0] = ( earth[i1][i0-1] - earth[i1][i0] ) / 
		     ( earth[i1][i0-1] + earth[i1][i0] );
    }}
    for (  int i0 =  0;   i0 < n0;    i0++) {        // tapering the edges // 
      for (int i1 =  0;   i1 < 10;    i1++) 
	refl[i1][i0] = refl[i1][i0] * (i1+1 ) / 10.f; 
      for (int i1 = n1-1; i1 > n1-11; i1--) 
	refl[i1][i0] = refl[i1][i0] * (n1-i1) / 10.f; 
    }
    for (int i1 = 0; i1 < n1; i1++) {
      for (int i0 = 0;    i0 <  5;   i0++) 
	refl[i1][i0] = refl[i1][i0] * (i0+1 ) / 5.f; 
      for (int i0 = n0-1; i0 < n0-6; i0--) 
	refl[i1][i0] = refl[i1][i0] * (n0-i0) / 5.f;
    }
    ((Rsf2) modl).d = refl; 
  }
}

