package  geom.simple;

import   sep.io.*;
import   rsf.vector.*;
import  java.util.Random;

public class TraceGenerator {
  final static int GAUSSIAN  = 101;
  final static int IMPEDANCE = 102;
  int       type      = GAUSSIAN;
  RsfSpace  space; 
  Random    randy;

  public TraceGenerator(RsfSpace space) {
    this(new Random(1901), space); 
  }
  public TraceGenerator(Random randy, RsfSpace space) {
    this.space = space; 
    this.randy = randy;
  }
  TraceGenerator(TraceGenerator src) {
    this.type  = src.type; 
    this.space = (RsfSpace) src.space.copy();
    this.randy = src.randy;        // shallow copy!!! 
  }
  /** Should this return a jam.JamObject?? */
  public TraceGenerator copy() { return new TraceGenerator(this); }

  public void setSeed(long seed) {
    randy.setSeed(seed);
  }
  public void setTraceType(int type) {
    if (type > TraceGenerator.IMPEDANCE || type < TraceGenerator.GAUSSIAN) 
      System.err.println("Error: type unknown");
    this.type = type;       
  }
  /** usage: RsfFactory.newRsf(TraceGenerator.getLength(spc.getNSamples())) */
  public static float getLength(int[] nsamples) {
    double tracelength = 0.;
    for (int i=0; i < nsamples.length; i++) {
      tracelength += nsamples[i] * nsamples[i];
    }
    return (int) (6 * Math.sqrt(tracelength));
  }
  public Rsf createTrace() {
    Rsf1 trace = (Rsf1) space.newMember(); 
    switch(type) {
    case TraceGenerator.GAUSSIAN: 
      return createGaussianTrace(trace);
    case TraceGenerator.IMPEDANCE:
      return createImpedanceTrace(trace);
    default: 
      System.err.println("Error: Unknown type.");
      return null; 
    }
  }
  public Rsf createGaussianTrace(Rsf1 trace) { 
    float[] tt = trace.d;
    for (int i=0; i < tt.length; i++) 
      tt[i] = (2.f * randy.nextFloat() - 1.f); 
    return trace;
  }
  public Rsf createImpedanceTrace(Rsf1 trace) { 
    float[] tt = trace.d;          tt[0] = 1.f;
    for (int i=1; i < tt.length; i++) {
      if (randy.nextFloat() < 0.2) tt[i] = tt[i-1]; 
      else                         tt[i] = 1.f + .1f * randy.nextFloat();
    }
    return trace;
  }
  /** @param nsamples=100 number of samples in trace */
  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args); 
    int nsamples = cmd.getInt("nsamples", 100); 
    TraceGenerator tgtor = new TraceGenerator(new RsfSpace(nsamples)); 
    Rsf trace = tgtor.createTrace();
    System.err.println(trace); 
    if (cmd.containsKey("out")) trace.write(cmd.getString("out"));
    else                        trace.write(System.out);        
  }
} 


