package  miss.bisolver;

import   conv.convi.*;
import    rsf.vector.*;
import    jam.operator.*;
import    jam.vector.*;
import  juice.solver.*;
import  sep.io.ArgumentKeeper;

public class LinearVersion {
  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);

    String  inRsf  = argKeeper.getString("in", "../../Data/hole.HH");
    String outRsf  = argKeeper.getString("out",  "infill.H");
    int    niter   = argKeeper.getInt   ("niter", "30");
    float  tol     = argKeeper.getFloat ("tol",   "1.0e-13F");

    Rsf data = RsfFactory.newRsf(inRsf);
    Rsf filt = getPefKnown();

    InternalConvolution
      conv = new InternalConvolution(InternalConvolution.kInput,
				     (RsfSpace) data.getSpace(),
				     InternalConvolution.kFilter,
				     (RsfSpace) filt.getSpace());

    BiLinMissDataEquations
      eqs = new BiLinMissDataEquations(conv, data, filt);
    NormalEqns nreq  = new NormalEqns(eqs.getA(), eqs.getb());
    //    CGSolverNL      cgSlv = new CGSolverNL(new IteratorSimple(tol, niter, true));
    CGNonlinSolver cgSlv = new CGNonlinSolver(
		               nreq.getAtA().getDomain(), 
			       nreq.getAtA().getRange(), 
			       new IteratorSimple(tol, niter, true)
			       );

    ProductVector x = (ProductVector) conv.getDomain().newMember();
    cgSlv.solve(nreq.getAtA(), nreq.getAtb(), x);

    x.getSubVector(0).add(data);
    x.getSubVector(1).add(filt);

    ((Rsf) x.getSubVector(0)).write(outRsf);
    ((Rsf) x.getSubVector(1)).write("filter.H");
  }

  static Rsf getPefKnown() {
    Rsf2 pefKnown = (Rsf2) RsfFactory.newRsf(5, 3);
    pefKnown.fill(0F);
    pefKnown.d[0][2] = 1F;
    return pefKnown;
  }
}
