package  miss.bisolver;

import   conv.convi.*;
import    rsf.vector.*;
import    jam.operator.*;
import    jam.vector.*;
import  juice.solver.*;
import  juice.operator.*;
import    sep.operator.SepBiLinearOperator;
import  sep.io.ArgumentKeeper;

public class NonLinearVersion {
  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);

    String  inRsf  = argKeeper.getString("in", "../../Data/hole.HH");
    String outRsf  = argKeeper.getString("out",  "infill.H");
    int    niter   = argKeeper.getInt   ("niter", "30");
    float  tol     = argKeeper.getFloat ("tol",   "1.0e-13F");

    Rsf data = RsfFactory.newRsf(inRsf);
    Rsf filt = getPefKnown();

    InternalConvolution
      conv = new InternalConvolution(InternalConvolution.kInput,
				     (RsfSpace) data.getSpace(),
				     InternalConvolution.kFilter,
				     (RsfSpace) filt.getSpace());

    BiLinMissDataEquations
      eqs = new BiLinMissDataEquations(conv, data, filt);
    NormalEqns nreq  = new NormalEqns(eqs.getA(), eqs.getb());
    CompBiLinOpHess O = new CompBiLinOpHess(conv,
					    eqs.getFreeMask1(),
					    eqs.getFreeMask2());
    CGSolverNL  cgSlv =
      new CGSolverNL(new IteratorSimple(tol, niter, true),
		     new CompoundOp(O, eqs.getA().adjoint()));

    ProductVector x = (ProductVector) conv.getDomain().newMember();
    cgSlv.solve(nreq.getAtA(), nreq.getAtb(), x);

    x.getSubVector(0).add(data);
    x.getSubVector(1).add(filt);

    ((Rsf) x.getSubVector(0)).write(outRsf);
    ((Rsf) x.getSubVector(1)).write("filter.H");
  }

  static Rsf getPefKnown() {
    Rsf2 pefKnown = (Rsf2) RsfFactory.newRsf(5, 3);
    pefKnown.fill(0F);
    pefKnown.d[0][2] = 1F;
    return pefKnown;
  }
}

class CompBiLinOpHess extends SepBiLinearOperator {
  BiLinearOperatorHessian B;
  hasHessian              L1, L2;

  /**
   * @param L1 must be hasAdjoint
   * @param L2 must be hasAdjoint
   */
  public CompBiLinOpHess(BiLinearOperatorHessian B,
			 hasHessian L1, hasHessian L2) {
    super(B.getDomain1(), B.getDomain2(), B.getRange());
    this.B  = B;
    if (! (L1 instanceof hasAdjoint)) {
      System.err.println("Error in CompBiLinOpHess constructor: " +
			 "L1 is not a hasAdjoint.");
    }
    this.L1 = L1;
    if (! (L2 instanceof hasAdjoint)) {
      System.err.println("Error in CompBiLinOpHess constructor: " +
			 "L2 is not a hasAdjoint.");
    }
    this.L2 = L2;
  }

  protected void apply(boolean add, Vector in1, Vector in2,
		       Vector out, int result) {
    if (add)
      switch (result) {
      case SepBiLinearOperator.kInput1:
	((hasAdjoint) L1).adjoint().
	  image(((hasAdjoint) B.getOp2(L2.image(in2))).adjoint().image(out),
		in1);
	break;
      case SepBiLinearOperator.kInput2:
	((hasAdjoint) L2).adjoint().
	  image(((hasAdjoint) B.getOp1(L1.image(in1))).adjoint().image(out),
		in2);
	break;
      case SepBiLinearOperator.kOutput:
	B.imageBinary(L1.image(in1), L2.image(in2), out);
	break;
      }
    else
      switch (result) {
      case SepBiLinearOperator.kInput1:
	((hasAdjoint) L1).adjoint().
	  addImage(((hasAdjoint) B.getOp2(L2.image(in2))).adjoint().image(out),
		   in1);
	break;
      case SepBiLinearOperator.kInput2:
	((hasAdjoint) L2).adjoint().
	  addImage(((hasAdjoint) B.getOp1(L1.image(in1))).adjoint().image(out),
		   in2);
	break;
      case SepBiLinearOperator.kOutput:
	B.addImageBinary(L1.image(in1), L2.image(in2), out);
	break;
      }
  }

  public jam.JamObject copy() {
    System.err.println("CompBiLinOpHess.copy() : not yet implemented.");
    return null;
  }
}
