package  pef.pef;

import   jam.vector.*;
import   rsf.vector.*;
import   rsf.solver.*;
import   conv.convi.*;
import  juice.solver.*;
import   sep.operator.*;
import   sep.io.ArgumentKeeper;

/**
 * Finds a Prediction Error filter given a data volume. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class IcaiPef extends SepLinearOperator {
  Pef      pefi;
  int      iter;

  public IcaiPef(RsfSpace iputSpace, 
	         RsfSpace oputSpace, 
		 RsfSpace pefiSpace, int iter){
    super(iputSpace, oputSpace); 
    this.pefi = new Pef(iputSpace, pefiSpace, iter);
    this.iter = iter;
  }
  protected IcaiPef(IcaiPef src) {
    super(src);
    this.pefi = (Pef)  src.pefi.copy();
    this.iter = src.iter; 
  }
  public jam.JamObject copy() { return new IcaiPef(this);}

  protected void apply(boolean add, Vector iput, Vector oput) {
    Rsf filt = (Rsf) pefi.getRange().newMember();
    pefi.image(iput, filt);
    Icai conv = 
      new IcaiFactory(filt).getIcaiFromInputSpace((RsfSpace) this.getDomain());
    if (! add) { if (isFwd) oput.zero(); else iput.zero();}; 
    if (isFwd) conv.          addImage(iput, oput);
    else       conv.adjoint().addImage(oput, iput);
  }

  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    Rsf iput           = RsfFactory.newRsf(System.in); 
    int iter           = cmd.getInt("iter", 
                                 Pef.getNIterations(cmd.getIntArray("pef")));
    RsfSpace pefSpace  = new RsfSpace(cmd.getIntArray("pef"));

    IcaiPefFactory convFac = new IcaiPefFactory(pefSpace, iter); 
    IcaiPef        conv    = (IcaiPef)convFac.getOpWithDomain(iput.getSpace());
    Rsf            oput    = (Rsf)    conv.getRange().newMember();
    conv.image(iput, oput);

    oput.write(System.out);
  }
}
