package  pef.pef;

import   jam.vector.*;
import   rsf.vector.*;
import   rsf.solver.*;
import   conv.convi.*;
import    jam.solver.*;
import  juice.solver.*;
import   sep.operator.*;
import   sep.io.ArgumentKeeper;

/**
 * Finds a Prediction Error filter given a data volume. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class Pef extends SepOperator {
  int iter; 

  public Pef(RsfSpace iputSpace, RsfSpace oputSpace, int iter) {
    super(iputSpace, oputSpace);
    this.iter = iter; 
  }
  public Pef(RsfSpace iputSpace, RsfSpace oputSpace) {
    this(iputSpace, oputSpace, Pef.getNIterations(oputSpace.getNSamples()));
  }
  protected Pef(Pef src) {
    super(src);
    this.iter = src.iter; 
  }
  public jam.JamObject copy() { return new Pef(this);}

  /** This does not implement <tt>add</tt>. */
  protected void apply(boolean add, Vector iput, Vector oput) {
    RsfSpace pefSpace = (RsfSpace) oput.getSpace();
    Rsf          rslt = (Rsf)      oput; 
    Rsf          data = (Rsf)      iput; 

    Rsf       pefKnown =             getPefKnown   (pefSpace) ; // 0 but 1 el 
    Mask        maskOp = new SepMask(getPefFreeMask(pefSpace)); // used 

    Icaf convOp = (new IcafFactory(data)).getIcafFromFilterSpace(pefSpace);

    MaskedEqns mdeq  = new MaskedEqns(convOp, pefKnown, maskOp);
    NormalEqns nreq  = new NormalEqns(mdeq.getAM(), mdeq.getNegAk());

    // I think I can do without! 
    // mdeq.zeroKnownComponents(rslt); // sets the known components zero. 

    CGSolver cgSlv = new CGSolver(new IteratorSimple(1.0e-10F, iter, false));
    cgSlv.solve(nreq.getAtA(), nreq.getAtb(), rslt);

    rslt.add(pefKnown);
  }

  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    Rsf data = RsfFactory.newRsf(System.in); 
    Rsf rslt = RsfFactory.newRsf(cmd.getIntArray("pef"));
    int iter = cmd.getInt("iter", getNIterations(cmd.getIntArray("pef")));

    Pef pefi = new Pef((RsfSpace)data.getSpace(), 
		       (RsfSpace)rslt.getSpace(), iter); 
    pefi.image(data, rslt);

    rslt.write(System.out);
    System.err.println("Output: " + rslt);
  }

  /** estimates the number of iterations needed for CG to estimate Pef */
  static int getNIterations(int[] shape) {
    int iter = 1; 
    for (int i=0; i<shape.length; i++) iter *= shape[i];
    return (iter / 2); 
  }

  static Rsf getPefFreeMask(RsfSpace pefSpace) {
    return getPefFreeMask(getPefNonZeroMask(pefSpace));
  }

  static Rsf getPefFreeMask(Rsf pefNonZeroMask) {
    Rsf pefFreeMask = null; int n0,n0h; int n1,n1h; 
    switch (pefNonZeroMask.getDimension()) {
    case 1: 
      pefFreeMask = (Rsf1) pefNonZeroMask.copy();
      ((Rsf1)pefFreeMask).setDatum(0,         0F);
      break; 
    case 2:
      pefFreeMask = (Rsf2) pefNonZeroMask.copy();
      n0 = ((RsfSpace)pefFreeMask.getSpace()).getNSamples(0); n0h = (n0-1)/2; 
      ((Rsf2)pefFreeMask).setDatum(0,n0h,     0F);
      break;
    case 3:
      pefFreeMask = (Rsf3) pefNonZeroMask.copy();
      n1 = ((RsfSpace)pefFreeMask.getSpace()).getNSamples(1); n1h = (n1-1)/2; 
      n0 = ((RsfSpace)pefFreeMask.getSpace()).getNSamples(0); n0h = (n0-1)/2; 
      ((Rsf3)pefFreeMask).setDatum(0,n1h,n0h, 0F);
      break;
    default:
      System.err.println("Error: pefNonZeroMask only supported up " +
                         "to 3 dimensions.\n");
      break;
    }
    return pefFreeMask;
  }
  static Rsf getPefKnown(RsfSpace pefSpace) {
    Rsf pefKnown = null; int n0,n0h; int n1,n1h; 
    switch (pefSpace.getDimension()) {
    case 1:
      pefKnown = (Rsf1) pefSpace.newMember();
      ((Rsf1)pefKnown).fill(0F);
      ((Rsf1)pefKnown).setDatum(0,         1F);
      break; 
    case 2:
      pefKnown = (Rsf2) pefSpace.newMember();
      n0 = pefSpace.getNSamples(0); n0h = (n0-1)/2; 
      ((Rsf2)pefKnown).fill(0F);
      ((Rsf2)pefKnown).setDatum(0,n0h,     1F);
      break; 
    case 3:
      pefKnown = (Rsf3) pefSpace.newMember();
      n1 = pefSpace.getNSamples(1); n1h = (n1-1)/2; 
      n0 = pefSpace.getNSamples(0); n0h = (n0-1)/2; 
      ((Rsf3)pefKnown).fill(0F);
      ((Rsf3)pefKnown).setDatum(0,n1h,n0h, 1F);
      break; 
    default:
      System.err.println("Error: pefKnown only supported up " +
                         "to 3 dimensions.\n");
      break;
    }
    return pefKnown;
  }
  static Rsf getPefNonZeroMask(RsfSpace pefSpace) {
    Rsf pefMask = null; int n0,n0h; int n1,n1h; 
    switch (pefSpace.getDimension()) {
    case 1:
      pefMask = (Rsf1) pefSpace.newMember();
      ((Rsf1)pefMask).fill(1F);
      break;
    case 2:
      pefMask = (Rsf2) pefSpace.newMember();
      n0 = pefSpace.getNSamples(0); n0h = (n0-1)/2; 
      ((Rsf2)pefMask).fill(1F);
      for (int i0=0; i0 < n0h; i0++) {
	((Rsf2)pefMask).setDatum(0,i0, 0F);
      }      
      break;
    case 3: 
      pefMask = (Rsf3) pefSpace.newMember();
      n1 = pefSpace.getNSamples(1); n1h = (n1-1)/2; 
      n0 = pefSpace.getNSamples(0); n0h = (n0-1)/2; 
      ((Rsf3)pefMask).fill(1F);
      for   (int i1=0; i1 < n1h; i1++) {
	for (int i0=0; i0 < n0 ; i0++) {
	((Rsf3)pefMask).setDatum(0,0,i0, 0F);
      }} 
      for   (int i0=0; i0 < n0h; i0++) {
	((Rsf3)pefMask).setDatum(0,n1h,i0, 0F);
      }      
      break;
    default:
      System.err.println("Error: pefMask only supported up " +
                         "to 3 dimensions.\n");
      break;
    }
    return pefMask;
  }
}
