package book.nmo;

import jam.vector.*;
import jam.operator.*;
import rsf.vector.*;
import sep.operator.SepOperator;

/**
 * Nmo (as a semilinear operator)
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * <BR><BR>The map is   Nmo(m,v) |---> d
 * <BR> where: <UL>
 * <LI> Nmo = semilinear normal moveout operator (linear with respect to m)
 * <LI> m = model time trace 
 * <LI> v = moveout velocity
 * <LI> o = moved out data
 * </UL>
 * <BR> 
 * (Should this be Nmo and stack?)
 * Should we have a SepSemiLinearOperator implementation 
 * from which we derive?
 * 
 * @author    Matt Schwab
 * @version   26 Feb 1997
 */
public class Nmo implements SemiLinearOperator {
  /**
   * these variables should probably be in SepSemiLinearOperator class
   */
  protected RsfSpace   nmoMdlSpc;
  protected RsfSpace   stkVelSpc;
  protected RsfSpace   cmpDatSpc;

  public Nmo(RsfSpace stkVelSpc, RsfSpace nmoMdlSpc, RsfSpace cmpDatSpc) {
    this.stkVelSpc = stkVelSpc;
    this.nmoMdlSpc = nmoMdlSpc;
    this.cmpDatSpc = cmpDatSpc;
  }
  public Nmo(Space  stkVelSpc,     Space  nmoMdlSpc,     Space  cmpDatSpc) {
    this((RsfSpace) stkVelSpc, (RsfSpace) nmoMdlSpc, (RsfSpace) cmpDatSpc);
  }

  /** 
   * this should move to SepSemiLinearOperator class (using domain1, etc)
   */
  public boolean equals( Object yourNmo ) {
    if (!( yourNmo instanceof Nmo ))
      return false;
    else
      /*
      return (    domain1.equals(((Nmo) yourNmo).domain1 )
	       && domain2.equals(((Nmo) yourNmo).domain2 )
	       && range  .equals(((Nmo) yourNmo).range   )
	       );
      */      
      return (    stkVelSpc.equals(((Nmo) yourNmo).stkVelSpc)
	       && nmoMdlSpc.equals(((Nmo) yourNmo).nmoMdlSpc)
	       && cmpDatSpc.equals(((Nmo) yourNmo).cmpDatSpc)
	       );
  }
  /** 
   * I had to put these in since the Binary class requires it
   * all this can go into a SepSemiLinearOperator class if we 
   * can agree on the various arguments. 
   *
   * I choose the "parameter" (velocity) to be first argument since it 
   * is a bit like a subscript to the operator. 
   */
  public       Operator        getOp1(Vector stkVel) {
    return getLinearOp((Rsf) stkVel); 
  }
  public       Operator        getOp2(Vector nmoMdl) {
    return getNonLinearOp((Rsf) nmoMdl); 
  }
  public NmoM                  getNonLinearOp(Rsf nmoMdl) {
    return new NmoM(stkVelSpc, nmoMdl,    cmpDatSpc);
  }
  public NmoV                  getLinearOp   (Rsf stkVel ) { 
    NmoV fwd = new NmoV(stkVel, nmoMdlSpc, cmpDatSpc, true );
    NmoV adj = new NmoV(stkVel, nmoMdlSpc, cmpDatSpc, false);
    fwd.setAdjoint(adj);
    adj.setAdjoint(fwd);
    return fwd;
  }

  /**
   * <STRONG>WARNING: not yet implemented</STRONG>
   * Joel, Do we want that? You can always execute getNmoV(stkVel).adjoint()
   */
  public LinearOperatorAdjoint getPartialAdjoint () {
    System.err.println("Error: InternalConvolution" +
			".getPartialAdjoint() not yet implemented.\n" );
    return null; 
  }

  public Space   getDomain1() {return nmoMdlSpc;}
  public Space   getDomain2() {return stkVelSpc;}
  public Space   getRange  () {return cmpDatSpc;}

  public Vector image   (Vector stkVel, Vector nmoMdl) {
    Rsf cmpDat = new Rsf(this.cmpDatSpc);
    apply(true, false,  (Rsf)   stkVel, (Rsf)  nmoMdl,        cmpDat);
    return cmpDat; 
  }
  public void   image	(Vector stkVel, Vector nmoMdl, Vector cmpDat) {
    apply(true, false,  (Rsf)   stkVel, (Rsf)  nmoMdl, (Rsf)  cmpDat); 
  }
  public void   addImage(Vector stkVel, Vector nmoMdl, Vector cmpDat) {
    apply(true, true ,  (Rsf)   stkVel, (Rsf)  nmoMdl, (Rsf)  cmpDat); 
  }

  /**
   *  Joel?:
   *  cmpDat2 = (cmpDat2-cmpDat1) + Nmo(nmoMdl, stkVel)
   */
  public void   residual(Vector nmoMdl,  Vector stkVel,
			 Vector cmpDat1, Vector cmpDat2) {
    cmpDat2.neg(cmpDat1);
    addImage(nmoMdl, stkVel, cmpDat2); 
  }

  // I/O
  public String toString() {
    return ("SemiLinearOperator Nmo:"    + "\n" +
	     "nmoMdlSpc (domain1):\n" + getDomain1() + "\n" +
	     "velStkSpc (domain2):\n" + getDomain2() + "\n" +
	     "cmpDatSpc   (range):\n" + getRange  () ); 
  }

  /**
   * I like to have the forward flag at the beginning with the add flag. 
   * Just like Jon. Any other reasons? 
   * Joel: Different in InternalConvolution.
   */
  /*
  static public void apply(boolean isFwd, boolean add, 
			   Rsf stkVel, Rsf nmoMdl, Rsf cmpDat) {
    if (isFwd) {
      System.err.println("isFwd: " + isFwd             + "\n" +
			 "input1:" + stkVel.getSpace() + "\n" +
			 "input2:" + nmoMdl.getSpace() + "\n" +
			 "output:" + cmpDat.getSpace() );
    }
    else {
      System.err.println("isFwd: " + isFwd             + "\n" +
			 "input1:" + stkVel.getSpace() + "\n" +
			 "input2:" + cmpDat.getSpace() + "\n" +
			 "output:" + nmoMdl.getSpace() );
    }
  }
  */

  static public void apply(boolean fwd, boolean add, 
			   Rsf stkVel, Rsf nmoMdl, Rsf cmpDat) {

System.err.println("I am in apply! \n\n");
    // The velocity function can never be adjoint output since it is 
    // non-linear. 
    if (!add) {
      if (fwd) {
	cmpDat.zero();
      }
      else {
	nmoMdl.zero();
      }
    }

    // for the moment stkVel is ignored and a constant velocity is used.
    // we need constant vel construtor, I guess. 
    // test: stkVelSpc.getAxis[0].NSample() == nmoMdlSpc.getAxis[0].NSample()
    float vel = 1.4F;

    Rsf     x  = nmoMdl; Rsf y  = cmpDat;
    float[] xx = x.d,        yy = y.d;

    int   nx =  ((RsfSpace) x.getSpace()).getNSamples( 1 );
    float dx = (((RsfSpace) x.getSpace()).getAxes()[1]).delta;
    float ox = (((RsfSpace) x.getSpace()).getAxes()[1]).offset; 
    int   nz =  ((RsfSpace) x.getSpace()).getNSamples( 0 );
    float dz = (((RsfSpace) x.getSpace()).getAxes()[0]).delta;
    float oz = (((RsfSpace) x.getSpace()).getAxes()[0]).offset; 

    for (int ix=0; ix < nx ; ix++) { float xs = (ox + dx * ix) * 1f/vel  ; 
    for (int iz=0; iz < nz ; iz++) { float zs =  oz + dz * iz; 
      float ts = (float) Math.sqrt( zs * zs + xs * xs ) + 1.e-20f;
      float wt = (float) (zs/ts * (1./Math.sqrt(ts))); 
      int   it = (int) (.5 + (ts - oz) / dz);
      if( it < nz ) {
	if(fwd) {
	  //y.setDatum(iz,ix,  y.getDatum(iz,ix) + x.getDatum(it,ix) * wt) ; 
	  yy[y.i(iz,ix)] += xx[x.i(it,ix)] * wt ;
	  yy[0] = 0F;
	}
	else {
	  //x.setDatum(it,ix,  x.getDatum(it,ix) + y.getDatum(iz,ix) * wt); 
	  xx[x.i(it,ix)] += yy[y.i(iz,ix)] * wt ;
	  xx[0] = 0F;
	}
      }
    }}
    //if(!fwd) 
    //System.err.println("Nmo1: sample 0: " + x.d[0] + "\n");
if(!fwd) {
float x2x = ((Rsf) nmoMdl).getDatum(0,0);
System.err.println("Nmo2: sample 0: " + x2x + "\n");
}
  }

  /**
   * these methods are nice once we have a title field in Rsf ... 
   * should I preserve the interface and make create binary? or both? 
   * I hate this proliferation of constructors. In the case of Nmo 
   * maybe not needed since the Spaces are arbitrary and therefore 
   * easily created.

  static RsfSpace createNmoMdlSpace ( RsfSpace cmpDatSpc ) {
    // change RsfSpace title .. 
    return new RsfSpace(cmpDatSpc); 
  }
  static RsfSpace createCmpDatSpace ( RsfSpace NmoMdlSpc ) {
    // change RsfSpace titel ..
    return new RsfSpace(NmoMdlSpc); 
  }
  static RsfSpace createStkVelSpace ( RsfSpace MdlOrDatSpc ) {
    // change RsfSpace titel ..
    Axis[] velAxis = {(MdlOrDatSpc.getAxes())[0]};
    return new RsfSpace(velAxis);
  }
  */

}






