package  proc.web;

import    jam.vector.*;
import    jam.operator.*;
import    rsf.vector.*;

/**
 *  Normal Moveout
 *
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author    Matt Schwab
 */
public class Nmo implements jam.operator.hasAdjoint {
  protected float vel; 

  /**
   * @param space filter or output space of the convolution
   * @param velocity (default 1.44 km/s)
   */
  public Nmo(RsfSpace domainAndRange, float velocity) {
    setInput (domainAndRange);
    setOutput(domainAndRange);
    setForward(true);
    vel    = velocity                            ;
  }

  // copy constructor
  Nmo(Nmo src) {
    this.vel = src.vel;
  }

  public void apply(boolean add, Vector in, Vector out) {
    Rsf2   x = (Rsf2) in;  float[][] xx = x.d;
    Rsf2   y = (Rsf2) out; float[][] yy = y.d;
    int   nx =  ((RsfSpace) x.getSpace()).getNSamples(1);
    float dx = (((RsfSpace) x.getSpace()).getAxes()[1]).delta;
    float ox = (((RsfSpace) x.getSpace()).getAxes()[1]).offset; 
    int   nz =  ((RsfSpace) x.getSpace()).getNSamples(0);
    float dz = (((RsfSpace) x.getSpace()).getAxes()[0]).delta;
    float oz = (((RsfSpace) x.getSpace()).getAxes()[0]).offset; 

    for (int ix=0; ix < nx ; ix++) { float xs = (ox + dx * ix) * 1f/vel  ; 
    for (int iz=0; iz < nz ; iz++) { float zs =  oz + dz * iz; 
      float ts = (float) Math.sqrt(zs * zs + xs * xs) + 1.e-20f;
      float wt = (float) (zs/ts * (1./Math.sqrt(ts))); 
      int   it = (int) (.5 + (ts - oz) / dz);
      if (it < nz) {
	if(isFwd)
	  yy[ix][iz] += xx[ix][it] * wt;
	else
	  xx[ix][it] += yy[ix][iz] * wt;
      }
    }}
  }

  public boolean equals(Object nmo2) {
    if (this == nmo2) return true;
    if (! (nmo2 instanceof Nmo)) return false;
    return (super.equals(nmo2) &&
	    this.vel == ((Nmo) nmo2).vel);
  }

  public jam.JamObject copy() { return new Nmo(this); }

  // I/O
  public String toString() {
    return ("Nmo (" + (isFwd ? "forward":"adjoint") + "):\n" +
	    "Space:\n" + getDomain());
  }
  protected Space           input;
  protected Space           output;
  protected boolean         isFwd = true;
  protected hasAdjoint      adj   = null;

  protected void  setInput  (Space input   ) { this.input  = input ; }
  protected void  setOutput (Space output  ) { this.output = output; }
  protected void  setForward(boolean isFwd ) { this.isFwd  = isFwd;  }
  protected void  setAdjoint(hasAdjoint adj) { this.adj    = adj;    }

  protected void  createAdjoint() {
    try {
      adj = (hasAdjoint) this.clone();
    }
    catch (Exception e) {
      System.err.println(e);
    }
    ((Nmo) adj).setForward(!this.isFwd);
    ((Nmo) adj).setAdjoint(this);
  }

  // ----------------------------------------------------------------------
  // ---------- The remaining methods are required by the superclass ------
  // ----------------------------------------------------------------------

  public Space   getDomain () { return isFwd ? input:output; }
  public Space   getRange  () { return isFwd ? output:input; }

  public Vector  image(Vector d) {
    Vector r = getRange().newMember();
    image(d, r);
    return r; }

  public void    image(Vector d, Vector r) {
    if (isFwd) apply(false, d, r);
    else       apply(false, r, d);
    }
  
  public void addImage(Vector d, Vector r) {
    if (isFwd) apply(true , d, r);
    else       apply(true , r, d); }

  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d, r); }

  public hasAdjoint     adjoint() {
    if (adj == null) createAdjoint();
    return adj;
  }

  public LinearOperator           getGradientAt  (Vector x) {
    return adjoint();
  }
  public hasAdjoint               getDerivativeAt(Vector x) {
    return this;
  }
}

